@@ -1,14 +1,18 @@
+import datetime
 import json
 import logging
 import os
 import sys
 import time
-from datetime import datetime
+from logging.handlers import TimedRotatingFileHandler
+import csv
 
-import jwt
 import requests
 from requests import HTTPError
 
+logger = logging.getLogger('Bb_rest_helper')
+logger.propagate = False
+
 # Get_Config
 # A convience class to get configuration values that we do not want to show
 # (or upload to github) (key, secret, url) from a json file.It also sets up
@@ -24,6 +28,9 @@ from requests import HTTPError
 
 class Get_Config():
 
+    logger = logging.getLogger('Bb_rest_helper')
+    logger.propagate = False
+
     # Initializes the class by taking the configuration file path
     # as an argument, a typical value would be "./config.json".
     def __init__(self, file_path: str):
@@ -31,9 +38,9 @@ class Get_Config():
         try:
             with open(self.file_path) as conf:
                 self.data = json.load(conf)
-                logging.info("Configuration file loaded")
+                logger.info("Configuration file loaded")
         except FileNotFoundError as e:
-            logging.error('No configuration file found - Exit program')
+            logger.error('No configuration file found - Exit program')
             sys.exit()
 
     # Returns url value from the configuration file to a variable
@@ -58,186 +65,86 @@ class Get_Config():
 
 class Auth_Helper():
 
+    logger = logging.getLogger('Bb_rest_helper')
+    logger.propagate = False
+
     # Initializes the auth helper by taking the target system url,
     # PI key and secret as arguments.
     def __init__(self, url: str, key: str, secret: str):
         self.url = url
         self.key = key
         self.secret = secret
+        self.learn_token = None
+
+    # Method that returns True when the token expires. Used by the learn_auth() method.
+    def token_is_expired(self, expiration_datetime):
+        self.expiration_datetime = expiration_datetime
+        self.time_left = (self.expiration_datetime -
+                          datetime.datetime.now()).total_seconds()
+        if self.time_left < 1:
+            time.sleep(1)
+            return True
+        else:
+            return False
 
     # Returns the authentication token for Blackboard Learn.
     def learn_auth(self):
-        try:
-            self.url_token = "/learn/api/public/v1/oauth2/token"
-            self.params = {"grant_type": "client_credentials"}
-            self.headers = {
-                'Content-Type': "application/x-www-form-urlencoded"}
-            r = requests.request(
-                "POST",
-                self.url +
-                self.url_token,
-                headers=self.headers,
-                params=self.params,
-                auth=(
-                    self.key,
-                    self.secret))
-            r.raise_for_status()
-            data = json.loads(r.text)
-            self.learn_token = data["access_token"]
-            logging.info("Learn Authentication successful")
-            logging.info("Token expires in: " + str(data["expires_in"]))
-            return self.learn_token
-        except requests.exceptions.HTTPError as e:
-            data = json.loads(r.text)
-            logging.error(data["error_description"])
-
-    # Returns the authentication token for Blackboard Collaborate.
-    def collab_auth(self):
-        self.token_url = '/token'
-        self.exp = int(round(time.time() * 1000)) + 270000
-        self.claims = {"iss": self.key, "sub": self.key, "exp": self.exp}
-        # Encode the JWT assertion with the jWT module, that includes claims
-        # and the secret.
-        self.assertion = jwt.encode(self.claims, self.secret)
-        self.credentials = 'urn:ietf:params:oauth:grant-type:jwt-bearer'
-        self.headers = {  # Content type is sent as a header
-            'Content-Type': 'application/x-www-form-urlencoded'
-        }
-        # Grant credentials and assertion are sent as parameters
-        self.params = {
-            'grant_type': self.credentials,
-            'assertion': self.assertion
-        }
-        try:
-            r = requests.request(
-                'POST',
-                self.url +
-                self.token_url,
-                params=self.params,
-                headers=self.headers,
-                auth=(
-                    self.key,
-                    self.secret))
-            r.raise_for_status()
-            data = json.loads(r.text)
-            logging.info('Collaborate Authentication successful')
-            logging.info("Token expires in: " + str(data["expires_in"]))
-            self.collab_token = data['access_token']
-            return self.collab_token
-        except requests.exceptions.HTTPError as e:
-            data = json.loads(r.text)
-            logging.error(data["error"])
-
-# A separate class to support Ally as a service requests.
-# it is separated as it uses different authentication and has
-# a still limited set of features
-
-
-class Ally_Helper():
-
-    # Initializes the auth helper by taking the target system url,
-    # PI key and secret as arguments.
-    def __init__(self, url: str, client_id: str, key: str, secret: str):
-        self.url = url
-        self.client_id = client_id
-        self.key = key
-        self.secret = secret
-
-    # Returns the authentication token for Blackboard Ally. Ally does not use
-    # a post call to an endpoint, instead, it encodes a jwt assertion that then
-    # is passed as a Bearer token.
-    def ally_auth(self):
-        self.iat = str(int(time.time()))
+        self.endpoint = "/learn/api/public/v1/oauth2/token"
+        self.params = {"grant_type": "client_credentials"}
         self.headers = {
-            'typ': 'JWT',
-            'alg': "Rs256"
-        }
-        self.claims = {
-            "clientId": self.client_id,
-            "iat": self.iat
-        }
-        self.assertion = jwt.encode(self.claims, self.secret).decode("utf-8")
-        self.ally_token = str(self.assertion)
-        return self.ally_token
+            'Content-Type': "application/x-www-form-urlencoded"}
 
-    # Uploads a file to ally service, getting the path to the file and the auth header
-    # arguments
-    def ally_upload_file(self, token: str, file_path: str):
-        self.file_path = file_path
-        self.token = token
-        self.headers = {
-            "Authorization": f'Bearer {self.token}'
-        }
-        self.ally_url = f'{self.url}/api/v2/clients/{self.client_id}/content'
-        self.files = {
-            'file': open(file_path, 'rb')
-        }
         try:
-            r = requests.request(
-                'POST',
-                self.ally_url,
-                files=self.files,
-                headers=self.headers)
-            r.raise_for_status()
-            data = json.loads(r.text)
-            logging.info(
-                'File uploaded to Ally, check processing status via ally_check_status()')
-            return data
-        except requests.exceptions.HTTPError as e:
-            logging.warning('An error occured during the request')
+            if self.learn_token == None:
 
-    # Returns the file hash to be used in the next requests from the upload
-    # request response
-    def ally_get_hash(self, upload_response: dict):
-        self.upload_response = upload_response
-        logging.info('Hash value extracted to a variable')
-        return self.upload_response["hash"]
+                r = requests.request(
+                    "POST",
+                    self.url +
+                    self.endpoint,
+                    headers=self.headers,
+                    params=self.params,
+                    auth=(
+                        self.key,
+                        self.secret))
+                r.raise_for_status()
+                self.data = json.loads(r.text)
+                self.learn_token = self.data["access_token"]
+                self.expires = self.data["expires_in"]
+                m, s = divmod(self.expires, 60)
+                self.now = datetime.datetime.now()
+                self.expires_at = self.now + \
+                    datetime.timedelta(seconds=s, minutes=m)
+                logger.info("Learn Authentication successful")
+                logger.info("Token expires at: " + str(self.expires_at))
+                return self.learn_token
 
-    # Checks the status of a file upload to Ally as a service, takes toekn and content hash
-    # as arguments.
-    def ally_check_status(self, token: str, content_hash: str):
-        self.content_hash = content_hash
-        self.token = token
-        self.headers = {
-            "Authorization": f'Bearer {self.token}'
-        }
-        self.check_url = f'{self.url}/api/v2/clients/{self.client_id}/content/{content_hash}/status'
-        try:
-            r = requests.request('GET', self.check_url, headers=self.headers)
-            r.raise_for_status()
-            data = json.loads(r.text)
-            logging.info('status checked. See response for details')
-            return data
-        except requests.exceptions.HTTPError as e:
-            logging.warning('An error occured during the request')
+            elif self.token_is_expired(self.expires_at):
+                logger.info('refresh token')
+                r = requests.request(
+                    "POST",
+                    self.url +
+                    self.endpoint,
+                    headers=self.headers,
+                    params=self.params,
+                    auth=(
+                        self.conf.get_key(),
+                        self.conf.get_secret()))
+                r.raise_for_status()
+                self.data = json.loads(r.text)
+                self.learn_token = self.data["access_token"]
+                self.expires = self.data["expires_in"]
+                m, s = divmod(self.expires, 60)
+                self.now = datetime.datetime.now()
+                self.expires_at = self.now + \
+                    datetime.timedelta(seconds=s, minutes=m)
+                logger.info("Learn Authentication successful")
+                logger.info("Token expires at: " + str(self.expires_at))
+                return self.learn_token
 
-    def ally_get_feedback(
-            self,
-            token: str,
-            content_hash: str,
-            feedback: bool = True):
-        self.content_hash = content_hash
-        self.token = token
-        self.feedback = feedback
-        self.feedback_url = f'{self.url}/api/v2/clients/{self.client_id}/content/{content_hash}'
-        self.headers = {
-            "Authorization": f'Bearer {self.token}'
-        }
-        self.params = {
-            "feedback": self.feedback
-        }
-        try:
-            r = requests.request(
-                'GET',
-                self.feedback_url,
-                headers=self.headers,
-                params=self.params)
-            r.raise_for_status()
-            data = json.loads(r.text)
-            logging.info('Feedback obtained, see response for details')
-            return data
         except requests.exceptions.HTTPError as e:
-            logging.error('An error occured during the request', exc_info=True)
+            data = json.loads(r.text)
+            logger.error(data["error_description"])
+
 
 # Bb_Requests
 # A class to simplify API calls to Blackboard REST APIs, provides functions
@@ -246,9 +153,14 @@ class Ally_Helper():
 
 class Bb_Requests():
 
+    logger = logging.getLogger('Bb_rest_helper')
+    logger.propagate = False
+
     # GET request. It takes a GET endpoint from the API, the authentication
     # token and a list of parameters as arguments. This request has been updated
-    # to support pagination
+    # to support pagination. Note this is currently a heavy method, use parameters
+    # limit responses if needed.
+
     def Bb_GET(
             self,
             base_url: str,
@@ -287,20 +199,17 @@ class Bb_Requests():
 
         except requests.exceptions.HTTPError as e:
             data = json.loads(r.text)
-            logging.error(data["message"])
+            logger.error(data["message"])
 
         except BaseException:
             # returns data from Learn REST API, all pages
-            logging.info("GET Request completed")
-            logging.info(f'API limit: {r.headers["X-Rate-Limit-Limit"]}')
-            logging.info(
+            logger.info("GET Request completed")
+            logger.info(f'API limit: {r.headers["X-Rate-Limit-Limit"]}')
+            logger.info(
                 f'Remaining API calls: {r.headers["X-Rate-Limit-Remaining"]}')
-            logging.info(
+            logger.info(
                 f'Time to reset API limit: {r.headers["X-Rate-Limit-reset"]}')
             return data_from_pages
-        finally:
-            # returns data from Collabrate API, as there is no pagination
-            return data_from_pages
 
     # POST request. It takes a POST endpoint from the API, the authentication token,
     # a list of parameters, and a json payload as arguments.
@@ -331,25 +240,21 @@ class Bb_Requests():
                 json=self.payload)
             data = json.loads(r.text)
             r.raise_for_status()
-            logging.info(f'API limit: {r.headers["X-Rate-Limit-Limit"]}')
-            logging.info(
+            logger.info(f'API limit: {r.headers["X-Rate-Limit-Limit"]}')
+            logger.info(
                 f'Remaining API calls: {r.headers["X-Rate-Limit-Remaining"]}')
-            logging.info(
+            logger.info(
                 f'Time to reset API limit: {r.headers["X-Rate-Limit-reset"]}')
-            logging.info("POST Request completed")
+            logger.info("POST Request completed")
             return data
         except requests.exceptions.HTTPError as e:
             data = json.loads(r.text)
-            logging.error(data["message"])
-        finally:
-            # Collaborate does not provide rate limit information, so just
-            # logging the request status and returning the data
-            logging.info("POST Request completed")
-            return data
+            logger.error(data["message"])
 
     # Uploads a file to the Blacboard Learn Api uploads endpoint, getting the path to the file and the auth header
     # arguments, it returns the file id that will be used in other calls to
     # the API (i.e. Creating content)
+
     def Bb_POST_file(self, base_url: str, token: str, file_path: str):
         self.base_url = base_url
         self.file_path = file_path
@@ -369,18 +274,18 @@ class Bb_Requests():
                 headers=self.headers)
             r.raise_for_status()
             data = json.loads(r.text)
-            logging.info(
+            logger.info(
                 'File uploaded to temporary storage, returning id')
-            logging.info("GET Request completed")
-            logging.info(f'API limit: {r.headers["X-Rate-Limit-Limit"]}')
-            logging.info(
+            logger.info("GET Request completed")
+            logger.info(f'API limit: {r.headers["X-Rate-Limit-Limit"]}')
+            logger.info(
                 f'Remaining API calls: {r.headers["X-Rate-Limit-Remaining"]}')
-            logging.info(
+            logger.info(
                 f'Time to reset API limit: {r.headers["X-Rate-Limit-reset"]}')
             return data['id']
         except requests.exceptions.HTTPError as e:
             data = json.loads(r.text)
-            logging.error(data["message"])
+            logger.error(data["message"])
 
     # PATCH request. It takes a PATCH endpoint from the API, the authentication token,
     # a list of parameters, and a json payload as arguments. A PATCH requests allows
@@ -412,21 +317,16 @@ class Bb_Requests():
                 json=self.payload)
             data = json.loads(r.text)
             r.raise_for_status()
-            logging.info(f'API limit: {r.headers["X-Rate-Limit-Limit"]}')
-            logging.info(
+            logger.info(f'API limit: {r.headers["X-Rate-Limit-Limit"]}')
+            logger.info(
                 f'Remaining API calls: {r.headers["X-Rate-Limit-Remaining"]}')
-            logging.info(
+            logger.info(
                 f'Time to reset API limit: {r.headers["X-Rate-Limit-reset"]}')
-            logging.info("PATCH Request completed")
+            logger.info("PATCH Request completed")
             return data
         except requests.exceptions.HTTPError as e:
             data = json.loads(r.text)
-            logging.error(data["message"])
-        finally:
-            # Collaborate does not provide rate limit information, so just
-            # logging the request status and returning the data
-            logging.info("PATCH Request completed")
-            return data
+            logger.error(data["message"])
 
     # PUT request. It takes a PUT endpoint from the API, the authentication token,
     # a list of parameters, and a json payload as arguments. A PUT request is meant
@@ -457,22 +357,19 @@ class Bb_Requests():
                 json=self.payload)
             data = json.loads(r.text)
             r.raise_for_status()
-            logging.info(f'API limit: {r.headers["X-Rate-Limit-Limit"]}')
-            logging.info(
+            logger.info(f'API limit: {r.headers["X-Rate-Limit-Limit"]}')
+            logger.info(
                 f'Remaining API calls: {r.headers["X-Rate-Limit-Remaining"]}')
-            logging.info(
+            logger.info(
                 f'Time to reset API limit: {r.headers["X-Rate-Limit-reset"]}')
-            logging.info("PUT Request completed")
+            logger.info("PUT Request completed")
             return data
         except requests.exceptions.HTTPError as e:
             data = json.loads(r.text)
-            logging.error(data["message"])
-        finally:
-            # Collaborate does not provide rate limit information, so just
-            # logging the request status and returning the data
-            logging.info("PUT Request completed")
-            return data
-
+            logger.error(data["message"])
+        except json.decoder.JSONDecodeError as e:
+            pass
+                
     # DELETE request. It takes a DELETE endpoint from the API, the authentication token
     # and a list of parameters as arguments.
     def Bb_DELETE(
@@ -499,18 +396,18 @@ class Bb_Requests():
             # A successful DELETE request returns a 204 code meaning that the server has
             # fulfilled the request but that there is no content to return.
             r.raise_for_status()
-            logging.info(f'API limit: {r.headers["X-Rate-Limit-Limit"]}')
-            logging.info(
+            logger.info(f'API limit: {r.headers["X-Rate-Limit-Limit"]}')
+            logger.info(
                 f'Remaining API calls: {r.headers["X-Rate-Limit-Remaining"]}')
-            logging.info(
+            logger.info(
                 f'Time to reset API limit: {r.headers["X-Rate-Limit-reset"]}')
-            logging.info("DELETE Request completed")
+            logger.info("DELETE Request completed")
         except KeyError:
             # Collaborate does not provide rate limit information, so just
             # logging the request status and returning the data
-            logging.info("DELETE Request completed")
+            logger.info("DELETE Request completed")
         except requests.exceptions.HTTPError as e:
-            logging.error('The resource could not be deleted')
+            logger.error('The resource could not be deleted')
 
 # A set of convenience functions (logging, printing, checking courses...),
 # this will be extended over time.
@@ -518,26 +415,43 @@ class Bb_Requests():
 
 class Bb_Utils():
 
+    logger = logging.getLogger('Bb_rest_helper')
+    logger.propagate = False
+
     # Sets logging with default path to ./logs and default level of DEBUG.
-    def set_logging(self, path: str = './logs', level=logging.DEBUG):
+    def set_logging(self, path: str = './logs', level=logging.DEBUG, when='h', interval=1):
         self.path = path
         self.level = level
+        self.when = when
+        self.interval = interval
         try:
             os.mkdir(self.path, 0o777)
-            logging.basicConfig(
-                format='%(asctime)-15s %(name)-22s %(funcName)-15s %(levelname)-8s %(message)s',
-                filename=f'{self.path}/Bb_helper_log_{datetime.now()}',
-                filemode="w",
-                level=self.level)
-            logging.info('Logs folder created')
-            logging.info('Logging has been set up')
+            logger = logging.getLogger('Bb_rest_helper')
+            logger.propagate = False
+            logger.setLevel(self.level)
+            handler = TimedRotatingFileHandler(f'{self.path}/Bb_rest_helper_log',
+                                               when=self.when,
+                                               interval=self.interval,
+                                               backupCount=5)
+            formatter = logging.Formatter(
+                '%(asctime)-15s %(name)-22s %(funcName)-15s %(levelname)-8s %(message)s')
+            handler.setFormatter(formatter)
+            logger.addHandler(handler)
+            logger.info('Logs folder created')
+            logger.info('Logging has been set up')
         except FileExistsError:
-            logging.basicConfig(
-                format='%(asctime)-15s %(name)-22s %(funcName)-15s %(levelname)-8s %(message)s',
-                filename=f'{self.path}/Bb_helper_log_{datetime.now()}',
-                filemode="w",
-                level=self.level)
-            logging.info('Logging has been set up')
+            logger = logging.getLogger('Bb_rest_helper')
+            logger.propagate = False
+            logger.setLevel(self.level)
+            handler = TimedRotatingFileHandler(f'{self.path}/Bb_rest_helper_log',
+                                               when=self.when,
+                                               interval=self.interval,
+                                               backupCount=5)
+            formatter = logging.Formatter(
+                '%(asctime)-15s %(name)-22s %(funcName)-15s %(levelname)-8s %(message)s')
+            handler.setFormatter(formatter)
+            logger.addHandler(handler)
+            logger.info('Logging has been set up')
 
     # Prints the response from any of the above methods in a prettified format
     # to the console.
@@ -546,41 +460,30 @@ class Bb_Utils():
         self.sort_keys = sort_keys
         if data:
             print(json.dumps(self.data, indent=4, sort_keys=self.sort_keys))
-            logging.info("Results printed to the console.")
+            logger.info("Results printed to the console.")
         else:
-            logging.warning("No data to print.")
+            logger.warning("No data to print.")
 
-    # Checks if a given Learn course exists in the server
-    def check_course_id(self, token: str, external_course_id: str):
+    # Checks if a given Learn course exists in the server. Takes the external course id as an argument
+    def check_course_id(self,url:str, token: str, external_course_id: str):
+        self.url = url
         self.token = token
         self.external_course_id = external_course_id
-        try:
-            endpoint_courses = "/learn/api/public/v3/courses"
-            headers = {
-                'Authorization': f'Bearer {self.token}',
-                'Content-Type': "Application/json"
-            }
-            params = {
-                "externalId": self.external_course_id,
-                "fields": "id"
-            }
-            r = requests.request(
-                'GET',
-                f'{self.url}{endpoint_courses}',
-                headers=headers,
-                params=params)
-            r.raise_for_status()
-            data = json.loads(r.text)
-            if data["results"]:
-                logging.info('The course has been found in the server.')
-                return True
-            else:
-                logging.warning(
-                    'The course could not be found, please check that the provided course id is the external id')
-                return False
-        except requests.exceptions.HTTPError as e:
-            data = json.loads(r.text)
-            logging.error(data["message"])
+        self.reqs = Bb_Requests()
+        self.endpoint_courses = "/learn/api/public/v3/courses"
+        self.params = {
+            "externalId": self.external_course_id,
+            "fields": "id"
+        }
+        data = self.reqs.Bb_GET(self.url, self.endpoint_courses, self.token, self.params)
+        print(data)
+        if data:
+            logger.info('The course has been found in the server.')
+            return True
+        else:
+            logger.warning('The course could not be found, please check that the provided course id is the external id')
+            return False
+
 
     # This method is provided to facilitate date formatting when importing dates from
     # other applications, i.e, dates in excel format, it can take a date string with
@@ -592,29 +495,29 @@ class Bb_Utils():
     # can be provided. The outcome of the method is YYYY-MM-DDTHH:MM:SS:000Z
     def time_format(
             self,
-            date: str,
+            dt: str,
             date_delimiter: str = "/",
             hour_delimiter: str = ":"):
-        self.date = date
+        self.dt = dt
         self.date_delimiter = date_delimiter
         self.hour_delimiter = hour_delimiter
-        if len(date) < 11:
-            split_date = date.split(self.date_delimiter)
-            date_formatted = datetime(int(split_date[2]), int(split_date[1]), int(
+        if len(self.dt) < 11:
+            split_date = dt.split(self.date_delimiter)
+            date_formatted = datetime.datetime(int(split_date[2]), int(split_date[1]), int(
                 split_date[0])).strftime('%Y-%m-%dT%H:%M:%S.%f')[:-3] + 'Z'
             return date_formatted
-        elif len(date) > 11 and len(date) < 17:
-            split_date = date.split(self.date_delimiter)
+        elif len(self.dt) > 11 and len(self.dt) < 17:
+            split_date = self.dt.split(self.date_delimiter)
             split_year = split_date[2].split()
             split_hour = split_year[1].split(self.hour_delimiter)
-            date_formatted = datetime(int(split_year[0]), int(split_date[1]), int(split_date[0]), int(
+            date_formatted = datetime.datetime(int(split_year[0]), int(split_date[1]), int(split_date[0]), int(
                 split_hour[0]), int(split_hour[1])).strftime('%Y-%m-%dT%H:%M:%S.%f')[:-3] + 'Z'
             return date_formatted
         else:
-            split_date = date.split(self.date_delimiter)
+            split_date = self.dt.split(self.date_delimiter)
             split_year = split_date[2].split()
             split_hour = split_year[1].split(self.hour_delimiter)
-            date_formatted = datetime(int(split_year[0]), int(split_date[1]), int(split_date[0]), int(
+            date_formatted = datetime.datetime(int(split_year[0]), int(split_date[1]), int(split_date[0]), int(
                 split_hour[0]), int(split_hour[1]), int(split_hour[2])).strftime('%Y-%m-%dT%H:%M:%S.%f')[:-3] + 'Z'
             return date_formatted
 
@@ -632,37 +535,26 @@ class Bb_Utils():
         self.token = token
         self.external_id = external_id
         self.final_id = final_id
-        self.url_courses = f'{self.url}/learn/api/public/v3/courses'
-        headers = {
-            'Authorization': f'Bearer {self.token}',
-            'Content-Type': "Application/json"
-        }
-        params = {
+        self.reqs = Bb_Requests()
+        self.endpoint_courses = '/learn/api/public/v3/courses'
+        self.params = {
             "externalId": self.external_id,
             "fields": self.final_id
         }
-        try:
-            r = requests.request(
-                'GET', self.url_courses, headers=headers, params=params)
-            r.raise_for_status()
-            data = json.loads(r.text)
-            if len(data) == 1:
-                logging.info("Course externalId converted to course Id")
-                return data["results"][0][self.final_id]
-            else:
-                logging.warning(
-                    "Several results have been found, please use a more specific Id")
-        except requests.exceptions.HTTPError as e:
-            data = json.loads(r.text)
-            logging.error(data["message"])
+        self.data = self.reqs.Bb_GET(self.url,self.endpoint_courses,self.token, self.params)
+        if len(self.data) == 1:
+            logger.info("Course externalId converted to course Id")
+            return self.data[0][self.final_id]
+        else:
+            logger.warning( "Several results have been found, please use a more specific Id")
 
 
-# A convenience method that further abstracts the setup and authentication process to return the token and the url in
-# just one line. This method is just for Learn and collaborate. The url has been added to avoid having to hardcode
-# this value or having to call Get_Config separately.
 
+    # A convenience method that further abstracts the setup and authentication process to return the token and the url in
+    # just one line. This method is just for Learn and collaborate. The url has been added to avoid having to hardcode
+    # this value or having to call Get_Config separately.
 
-    def quick_auth(self, filepath: str, platform: str):
+    def quick_auth(self, filepath: str, platform: str = 'Learn'):
         self.filepath = filepath
         self.platform = platform
         self.conf = Get_Config(self.filepath)
@@ -677,13 +569,50 @@ class Bb_Utils():
                 'url': self.url
             }
             return data
-        elif self.platform == "Collaborate":
-            self.token = self.auth.collab_auth()
-            data = {
-                'token': self.token,
-                'url': self.url
-            }
-            return data
         else:
-            logging.error(
-                'Please specify a platform, valid values are Learn and Collaborate.')
+            logger.error(
+                'Collaborate methods have been removed from V3. "platform" argument ketp for compatibility with default value of "Learn"')
+
+    # A simple method that detects wheter there is conectivtiy, returns True if connected and
+    # false if not. This methods uses requests to test connection to a site (google by default )
+    def check_connection(self, host="https://www.google.com/",timeout=int(5) ):
+        self.host = host
+        self.timeout = timeout
+        try:
+            req = requests.get(self.host,timeout = self.timeout)
+            return True
+        except (requests.ConnectionError,requests.Timeout) as e:
+            logger.warning("No connection")
+            return False
+    
+    #This method reads data from a csv file, it takes the path from the file, and optionally a delimiter
+    #(default ',') 
+    def read_csv(self, path, delimiter = ','):
+        self.delimiter = delimiter
+        self.path = path
+        self.data = []
+        try:
+            with open(self.path,'r',newline='') as csvfile:
+                self.reader = csv.DictReader(csvfile,delimiter=self.delimiter)
+                for self.row in self.reader:
+                    self.data_to_append = self.row
+                    self.data.append(self.data_to_append)
+                return self.data
+        except FileNotFoundError:
+            logger.error("File not found")
+    
+    def write_csv(self,path, headers, data, delimiter=','):
+        self.path = path
+        self.headers= headers
+        self.data= data
+        self.delimiter = delimiter
+        if os.path.isfile(self.path):
+            with open(self.path,'a',newline = '')as csvfile:
+                self.writer = csv.DictWriter(csvfile,fieldnames=self.headers, delimiter=self.delimiter)
+                self.writer.writerow(data)  
+        else:
+            with open(self.path,'a',newline = '')as csvfile:
+                self.writer = csv.DictWriter(csvfile,fieldnames=self.headers, delimiter=self.delimiter)
+                self.writer.writeheader()
+                self.writer.writerow(data)
+        
\ No newline at end of file
